
/* -- INCLUDE ------------------------------------------------------------- */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <iostream.h>
#include <string.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "address_class.h"

char * address_id = "$Id: address_class.C,v 1.2 1997/12/16 12:38:21 frank Exp $";

/* -- METHODS: address ---------------------------------------------------- */

c_address::c_address ()
{
  memset (&s_address, '\0', sizeof (sockaddr_in));

  s_addrlen                 = sizeof (sockaddr_in);
  s_address.sin_family      = AF_INET;
  s_address.sin_port        = htons (0);
  s_address.sin_addr.s_addr = htonl (INADDR_ANY);

  error = 0;
}

c_address::c_address (const u_short port)
{
  memset (&s_address, '\0', sizeof (sockaddr_in));

  s_addrlen                 = sizeof (sockaddr_in);
  s_address.sin_family      = AF_INET;
  s_address.sin_port        = htons (port);
  s_address.sin_addr.s_addr = htonl (INADDR_ANY);

  error = 0;
}

c_address::c_address (const u_long addr, const u_short port)
{
  memset (&s_address, '\0', sizeof (sockaddr_in));

  s_addrlen                 = sizeof (sockaddr_in);
  s_address.sin_family      = AF_INET;
  s_address.sin_port        = htons (port);
  s_address.sin_addr.s_addr = htonl (addr);

  error = 0;
}

c_address::c_address (const char *hostname, const u_short port)
{
  memset (&s_address, '\0', sizeof (sockaddr_in));

  s_address.sin_family = AF_INET;
  s_addrlen            = sizeof (sockaddr_in);
  s_address.sin_port   = htons (port);

  if ((s_address.sin_addr.s_addr = inet_addr (hostname)) == INADDR_NONE) {
    hostent *nodep;
    if ((nodep = gethostbyname (hostname)) == NULL) {
      ErrStr = "hostname unknown"; 
      error  = 1;
      return;
    }
    memcpy (&s_address.sin_addr.s_addr, nodep->h_addr, nodep->h_length);
  }
  error = 0;
  return;
}

c_address::c_address (const c_address & other)
{
  s_addrlen = other.s_addrlen;
  memcpy (&s_address, &other.s_address, sizeof (sockaddr_in));
  error = 0;
}

c_address::c_address (const char *hostname, 
		      const char *servname,
		      const char *proto)
{
  memset (&s_address, '\0', sizeof (sockaddr_in));

  servent *sp = getservbyname (servname, proto);
  if (sp == NULL) {
    ErrStr = "service unknown";
    error  = 1;
    return;
  }
  s_address.sin_family = AF_INET;
  s_address.sin_port   = sp->s_port;

  if ((s_address.sin_addr.s_addr = inet_addr (hostname)) == INADDR_NONE) {
    hostent *nodep;
    if ((nodep = gethostbyname (hostname)) == NULL) {
      ErrStr = "hostname unknown"; 
      error  = 1; 
      return;
    }
    memcpy (&s_address.sin_addr.s_addr, nodep->h_addr, nodep->h_length);
  }
  error = 0;
  return;
}


c_address::c_address (const char *servname, const char *proto)
{
  memset (&s_address, '\0', sizeof (sockaddr_in));
  
  servent *sp = getservbyname (servname, proto);
  if (sp == NULL) {
    ErrStr = "service unknown";
    error  = 1;
    return;
  }
  
  s_address.sin_family      = AF_INET;
  s_address.sin_addr.s_addr = htonl (INADDR_ANY);
  s_address.sin_port        = sp->s_port;

  error = 0;
  return;
}

int
c_address::islocal() const
{
  return (s_address.sin_addr.s_addr == INADDR_ANY);
}

const char *
c_address::gethostname() const
{
  static char hname[128];

  if (error)
    return NULL;

  if (islocal()) {
    ::gethostname (hname, 128);
    return hname;
  }

  hostent *hp = gethostbyaddr ((const char *) &s_address.sin_addr.s_addr, 4, 
			       AF_INET);
  return (hp != NULL) ? hp->h_name : ((const char *) NULL);
}     

const char *
c_address::getinetaddr() const
{
  const in_addr *addr;
  hostent *hp;

  if (islocal()) {
    if ((hp = gethostbyname (gethostname())) == NULL)
      return NULL;
    addr = (in_addr *) hp->h_addr;
  } 
  else 
    addr = &s_address.sin_addr;

  return inet_ntoa (*addr);
}

u_long 
c_address::getnetaddr() const
{
  hostent *hp;

  if (isremote())
    return ntohl (s_address.sin_addr.s_addr);

  if ((hp = gethostbyname (gethostname())) == NULL)
    return INADDR_NONE;

  return ntohl (*((long *)hp->h_addr));
}


ostream &operator<< (ostream &o, const c_address &addr)
{
  return o << addr.gethostname() << ":" << addr.getport();
} 
