/* ------------------------------------------------------------------------ */
/*
 *
 *  address_class.h
 *  
 *  $Id: address_class.h,v 1.3 1998/01/16 15:13:12 frank Exp $
 */

#ifndef ADDRESS_CLASS_H
#define ADDRESS_CLASS_H

/*
 * needed to understand these declarations
 */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#include <sys/types.h>
#include <netinet/in.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <iostream.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

/* -- CLASSES ------------------------------------------------------------- */

class c_address {
  friend class c_Socket;
  friend class c_socket_str;
  friend class c_socket_dgr;
  friend class c_socket_mc;

 public:
  /* constructors for remote addresses */
  c_address (const char *, const u_short = 0);
  c_address (const u_long, const u_short = 0);
  c_address (const char *, const char *, const char *);
  c_address (const c_address &);
  
  /* constructors for local addresses */ 
  c_address ();
  c_address (const u_short);
  c_address (const char *, const char *);
  
  int 			operator!(void)		const;
  int			fail(void)		const;
  const char *		whatswrong(void)	const;
  
  const char *		gethostname(void)	const;
  const char *		getinetaddr(void)	const;
  u_long		getnetaddr(void)	const;
  			operator const char *()	const;
  u_short		getport(void)		const;
  int			islocal(void)		const;
  int			isremote(void)		const;
  
  int			operator== (const c_address &) const;
  friend ostream &operator<< (ostream &, const c_address &);
  
 private:
  socklen_t s_addrlen;
  sockaddr_in s_address;
  
 protected:
  int error;
  const char *ErrStr;
};

extern ostream &operator<< (ostream &, const c_address &);

inline int
c_address::operator! () const
{
  return (error ? 1 : 0);
}

inline int
c_address::fail () const
{
  return error;
}

inline const char *
c_address::whatswrong () const
{
  return ErrStr;
}

inline c_address::operator const char *() const
{
  return gethostname();
}

inline u_short 
c_address::getport (void) const
{
  return ntohs (s_address.sin_port);
}

inline int 
c_address::isremote (void) const
{
  return !islocal ();
}

inline int
c_address::operator== (const c_address &other) const
{
  int flag=0;

  if (s_address.sin_family      == other.s_address.sin_family      &&
      s_address.sin_port        == other.s_address.sin_port        &&
      s_address.sin_addr.s_addr == other.s_address.sin_addr.s_addr)
    flag = 1;

  return flag;
}

#endif /* ADDRESS_CLASS_H */
