/*
 * Praktische Shared Memory-Klasse
 */

#include "fpconfig.h"

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdlib.h>
#include <errno.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <fpshmem.h>
#include "ipcbase.h"
#include "fpmode.h"

char * fpshmem_id = "$Id: fpshmem.C,v 1.1.1.1 1997/10/28 10:49:43 frank Exp $";

/*
 * Leerer Konstruktor
 */

fpShmem::fpShmem (void)
{
  shmAddress = (void *) -1;
  shmId      = -1;
}

/*
 * Copy Constructor
 */

fpShmem::fpShmem (const fpShmem &other)
{
  shmAddress = (void *) -1;
  shmId      = -1;

  if (other.shmId != -1) {
    attach (other.shmKey);
  }
}

/*
 * Kompletter Konstruktor
 */

fpShmem::fpShmem (key_t key, size_t thesize, int mode, int excl)
{
  shmAddress = (void *) -1;
  shmId      = -1;

  create (key, thesize, mode, excl);
}

/*
 * Wenn thesize==0, dann erzeugen wir einen Fehler, wenn das Segment noch
 *                  nicht existiert.
 * Wenn thesize!=0, dann schauen wir nach, ob das Segment bereits in aus-
 *                  reichender Groesse existiert. Wenn ja, dann geben wir
 *                  uns damit zufrieden. Wenn nein, dann legen wir es an.
 */

int
fpShmem::create (key_t key, size_t thesize, int mode, int excl)
{
  shmAddress = (void *) -1;
  shmKey     = key;
  seterror ();

  /*
   * wenn thesize>0, versuche, das shared Memory explizit anzulegen
   */

  if (thesize > 0 && mode != 0) {
    shmId = shmget (key, thesize, IPC_CREAT | IPC_EXCL | mode);

    if (shmId != -1) {
      /*
       * Okay, wir sind die ersten
       */
      if ((shmAddress = shmat (shmId, NULL, 0)) == (void *) -1) {
	shmctl (shmId, IPC_RMID, NULL);
	seterror (errno);
	shmId = -1;
	return -1;
      }
      arefirst = 1;
      seterror ();
      return 0;
    }

    /*
     * Gut, unser Versuch, das shared Memory exklusiv einzurichten, ist
     * fehlgeschlagen. Wahrscheinlich existiert die Resource bereits.
     */

    if (errno != EEXIST) {
      /*
       * Nein, das ist nicht das Problem.
       */
      seterror (errno);
      return -1;
    }

    if (excl) {
      /* heul, wir wollten aber selber initialisieren! */
      seterror (EEXIST);
      return -1;
    }
  }

  /*
   * Okay, haengen wir uns an das existierende shared Memory mit dran.
   * Beachte, dass der Aufruf fehlschlaegt, wenn das existierende shMem
   * kleiner als thesize ist.
   */

  if ((shmId = shmget (key, thesize, 0)) == -1) {
    seterror (errno);
    return -1;
  }

  if ((shmAddress = shmat (shmId, NULL, 0)) == (void *) -1) {
    seterror (errno);
    shmId = -1;
    return -1;
  }

  arefirst = 0;
  seterror ();
  return 0;
}

/*
 * Destruktor.
 */

fpShmem::~fpShmem ()
{
}

/*
 * Zuweisungsoperator
 */

fpShmem &
fpShmem::operator= (const fpShmem &other)
{
  detach ();

  if (other.shmId != -1) {
    attach (other.shmKey);
  }

  return *this;
}

/*
 * Operationen
 */

size_t
fpShmem::size (void)
{
  struct shmid_ds buf;

  if (shmctl (shmId, IPC_STAT, &buf) == -1) {
    seterror (errno);
    return 0;
  }

  return (size_t) buf.shm_segsz;
}

int
fpShmem::destroy (void)
{
  if (shmId == -1) {
    seterror (0);
    return 0;
  }

  shmdt ((char *) shmAddress);

  if (shmctl (shmId, IPC_RMID, NULL) == -1) {
    seterror (errno);
    return -1;
  }
  shmAddress = (void *) -1;
  shmId   = -1;
  seterror ();
  return 0;
}

int
fpShmem::detach (void)
{
  if (shmAddress != (void *) -1) {
    shmdt ((char *) shmAddress);
  }
  shmAddress = (void *) -1;
  shmId      = -1;
  seterror ();
  return 0;
}
