/*
 * Implementation einer etwas komfortableren Shared Memory-Klasse
 *
 * $Id: fpshmem.h,v 1.1.1.1 1997/10/28 10:49:44 frank Exp $
 */

#ifndef __FPSHMEM_H__
#define __FPSHMEM_H__

#include "fpconfig.h"

#include <sys/types.h>
#include <sys/ipc.h>
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#include "ipcbase.h"
#include "fpmode.h"

/*
 * Klassendeklaration
 */

class fpShmem : virtual public cIPC {
public:
  fpShmem (key_t, size_t = 0, int = 0, int = 0);
  fpShmem (const fpShmem &);
  fpShmem (void);

  /*
   * Verwaltung
   */

  int attach  (const fpShmem *);
  int attach  (key_t, size_t = 0);
  int create  (key_t, size_t, int = S_IRWXU, int = 0);
  int detach  (void);
  int destroy (void);

  fpShmem & operator= (const fpShmem &);

  /*
   * Operationen
   */

  void * access  (void) const;	/* get shmem region                 */
  int    first   (void) const;	/* were we the first?               */
  size_t size    (void);	/* size of shmem region             */

  operator char*() const;	/* returns address of shared memory */

  virtual ~fpShmem ();

protected:
  key_t  getkey (void) const;   /* key abfragen                     */

private:
  int    arefirst;
  key_t  shmKey;		/* requested key                    */
  int    shmId;			/* shared memory identifier         */
  void * shmAddress;		/* address of attached shared mem   */
};

inline
fpShmem::operator char*() const
{
  return (char *) shmAddress;
}

inline void *
fpShmem::access (void) const
{
  return shmAddress;
}

inline int
fpShmem::attach (key_t key, size_t size)
{
  return create (key, size, 0, 0);
}

inline int
fpShmem::attach (const fpShmem * other)
{
  return attach (other->shmKey);
}

inline int
fpShmem::first (void) const
{
  return arefirst;
}

inline key_t
fpShmem::getkey (void) const
{
  if (shmId == -1) return 0;
  return shmKey;
}

#endif
