
#include "fpconfig.h"

#include <pthread.h>
#include "itc.h"

char * itc_id = "$Id: itc.C,v 1.2 1998/01/16 15:13:13 frank Exp $";

extern int pthread_mutexattr_settype (pthread_mutexattr_t *__attr,
				      int __kind);


#ifdef HAVE_POSIX_THREADS_1CD10
#ifndef HAVE_PTHREAD_MUTEXATTR_SETTYPE_NP
#define pthread_mutexattr_settype_np pthread_mutexattr_settype
#endif
#endif

c_mutex::c_mutex(){


#ifndef MIT
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutexattr_init(&m_attr);
#else
	pthread_mutexattr_create(&m_attr);
#endif
#else
	m_attr.m_type = MUTEX_TYPE_FAST;
#endif
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutex_init(&m_handle, &m_attr);
#else
	pthread_mutex_init(&m_handle, m_attr);
#endif

}

c_mutex::c_mutex(Mutex_Types mte){

#ifndef MIT
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutexattr_init(&m_attr);

	switch(mte){ 

	case DEBUGMUTEX:
		pthread_mutexattr_settype_np(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);
		break;

	case RECMUTEX:
		pthread_mutexattr_settype_np(&m_attr,PTHREAD_MUTEX_RECURSIVE_NP);
		break;
	}
#else
        pthread_mutexattr_create(&m_attr);

        switch(mte){

        case DEBUGMUTEX:
                pthread_mutexattr_setkind_np(&m_attr,MUTEX_NONRECURSIVE_NP);
		break;

        case RECMUTEX:
                pthread_mutexattr_setkind_np(&m_attr,MUTEX_RECURSIVE_NP);
		break;

        default:
                pthread_mutexattr_setkind_np(&m_attr,MUTEX_FAST_NP);
		break;

        }


#endif
#else
	switch(mte){

	case DEBUGMUTEX:
		m_attr.m_type = MUTEX_TYPE_DEBUG;
		break;

	case RECMUTEX:
		m_attr.m_type = MUTEX_TYPE_COUNTING_FAST;
		break;

	default:
		m_attr.m_type = MUTEX_TYPE_FAST;
		break;

	}
#endif
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutex_init(&m_handle, &m_attr);
#else
        pthread_mutex_init(&m_handle, m_attr);
#endif
}

c_mutex::~c_mutex(){

	pthread_mutex_destroy(&m_handle);

}

int c_mutex::lock(){

	pthread_mutex_lock(&m_handle);

}


int c_mutex::unlock(){

	pthread_mutex_unlock(&m_handle);

}

c_condition::c_condition(){

#ifndef MIT
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_condattr_init(&c_attr);
#else
        pthread_condattr_create(&c_attr);
#endif
#else	
	c_attr.c_type = COND_TYPE_FAST;
#endif
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_cond_init(&c_handle, &c_attr);
#else
         pthread_cond_init(&c_handle, c_attr);
#endif
}


c_condition::c_condition(pthread_condattr_t *cattr){

#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_cond_init(&c_handle, cattr);
#else
        pthread_cond_init(&c_handle, *cattr);
#endif

}

c_condition::~c_condition(){

	pthread_cond_destroy(&c_handle);

}

int c_condition::signal(){

	pthread_cond_signal(&c_handle);

}

int c_condition::broadcast(){

	pthread_cond_broadcast(&c_handle);

}

int c_condition::wait(c_mutex *conmutex){

	pthread_cond_wait(&c_handle, &(conmutex->m_handle));

}

c_section::c_section(){


#ifndef MIT
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_condattr_init(&c_attr);
#else
        pthread_condattr_create(&c_attr);
#endif
#else	
	c_attr.c_type = COND_TYPE_FAST;
#endif
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_cond_init(&c_handle, &c_attr);
#else
        pthread_cond_init(&c_handle, c_attr);
#endif
#ifndef MIT
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutexattr_init(&m_attr);
#else
        pthread_mutexattr_create(&m_attr);
#endif
#else
	m_attr.m_type = MUTEX_TYPE_FAST;
#endif
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_mutex_init(&m_handle, &m_attr);
#else
        pthread_mutex_init(&m_handle, m_attr);
#endif


}

c_section::~c_section(){

	pthread_mutex_destroy(&m_handle);
	pthread_cond_destroy(&c_handle);


}

int c_section::enter(){

	pthread_mutex_lock(&m_handle);
	
}

int c_section::wait(){

	pthread_cond_wait(&c_handle, &m_handle);

}

int c_section::signal(){

	pthread_cond_signal(&c_handle);

}

int c_section::broadcast(){

	pthread_cond_broadcast(&c_handle);

}

int c_section::leave(){

	pthread_mutex_unlock(&m_handle);

}







