/* ------------------------------------------------------------------------ */
/*
 *
 *  socket_class.C
 *
 *  compile:
    CC -c socket_class.C
 */
 

/* -- INCLUDE ------------------------------------------------------------- */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#ifdef TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "address_class.h"
#include "socket_class.h"

/*
 * HP/UX braucht mal wieder eine Extrawurst
 */

#ifdef __hpux__
#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#define FT_SETTYPE(s)	((int *) s)
#endif
#endif
#endif
#ifndef FT_SETTYPE
#define FT_SETTYPE(s)	(s)
#endif

/* -- VARIABLE ------------------------------------------------------------ */

static char *errormsg[] = {
  "read/write: no connect",
  "read/write: no accept",
  "cannot read: source unknown",
  "cannot write: destination unknown"
};

char * socket_id = "$Id: socket_class.C,v 1.5 1998/01/16 15:13:13 frank Exp $";

/* -- METHODS: Socket ----------------------------------------------------- */

c_Socket::c_Socket (const int sockdesc, const int err)
/*
 *  constructor is designed for method socket_str::accept(), because the
 *  UNIX system call accept() creates a new socket of the desired type. 
 *  The first parameter is returned by accept() and it is the socket 
 *  descriptor.
 */ 
{
  desc   = sockdesc;
  error  = err;
  ErrStr = NULL;
  setintrstatus (repeat);
}

	
c_Socket::c_Socket (const int comm_mode)
{
  if ((desc = socket(AF_INET, comm_mode, 0)) < 0) {
    error  = errno;
    ErrStr = strerror (errno);
  } 
  else {
    setintrstatus (repeat);
    error  = 0;
    ErrStr = NULL;
  }
}

const char *c_Socket::gethostname(void)
{
  static char hostname[MAXHOSTNAMELEN+1];
  static int status = ::gethostname (hostname, MAXHOSTNAMELEN);
  
  return (status == 0) ? hostname : ((const char *) NULL);
}


int c_Socket::getladdr (c_address &addr)
{
  if (getsockname (desc, (struct sockaddr *) &addr.s_address,
		   &addr.s_addrlen) < 0) {
    ErrStr     = strerror (error = errno);
    addr.error = 1;
    return -1;
  }
  
  return addr.error = 0;  
}


/* -- METHODS: c_socket_str ----------------------------------------------- */

c_socket_str::c_socket_str() : c_Socket (SOCK_STREAM)
{
  connected = 0;
  p_peer = NULL;
}

c_socket_str::c_socket_str (c_address &addr) : c_Socket (SOCK_STREAM)
{
  u_short isportudef = addr.s_address.sin_port == 0;
  int opt;
  
  p_peer = NULL;
  if (addr.isremote()) {
    return;
  }

  opt = 1;
  if (setsockopt (getdesc(), SOL_SOCKET, SO_REUSEADDR,
		  &opt, sizeof (int)) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }
  
  if (bind (getdesc(),
	    (struct sockaddr *) &addr.s_address,
	    addr.s_addrlen) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }
  
  if (isportudef)
    if (getsockname (getdesc(), 
		     (struct sockaddr *) &addr.s_address,
		     &addr.s_addrlen) < 0) {
      ErrStr = strerror (error = errno);
      return;
    }
  
  if (listen (getdesc(), 5) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }
  
  connected = 0;  
}  

c_socket_str::c_socket_str (const c_address &addr, 
			    const int sockdesc,
			    const int err)
	: c_Socket (sockdesc, err)
{
  connected = 1;
  p_peer    = new c_address (addr);
}

c_socket_str::c_socket_str (const c_socket_str & other)
  : c_Socket (other)
{
  if (other.p_peer) {
    p_peer = new c_address (*other.p_peer);
  } else {
    p_peer = NULL;
  }

  if ((connected = other.connected)) {
    setdesc (dup (other.getdesc()));
  }
}

c_socket_str::~c_socket_str()
{
  if (p_peer != NULL) delete p_peer;
}

c_socket_str *
c_socket_str::accept(void)
{
  c_address addr; 
  int sockdesc;
 
next:
  if ((sockdesc = ::accept (getdesc(),
			    (struct sockaddr *) &addr.s_address,
			    &addr.s_addrlen))<0) {
    if (errno == EINTR && getintrstatus()) goto next;
    ErrStr = strerror (error = errno);
    return NULL;
  }

  return new c_socket_str (addr, sockdesc, 0);
}
	

int c_socket_str::connect (const c_address &addr)
{
  int status = ::connect (getdesc(),
			  (struct sockaddr *) &addr.s_address,
			  addr.s_addrlen);
  
  if (status < 0)
    ErrStr = strerror (error = errno);
  else {
    connected = 1;
    p_peer = new c_address (addr);
  }
  return status;
}


int c_socket_str::read (void *buf, const int len)
{
  int n_read;
  
  if (!connected) {
    ErrStr = errormsg[0];
    return (error = -1);
  }
  
next:
  n_read = ::read (getdesc(), (char *) buf, len);
  if (n_read < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  } 
  else if (n_read == 0) {
    ErrStr = "no data read; possibly lost connection";
    error  = 1;
  }
  
  return n_read;
}

int c_socket_str::peek (void *buf, const int len)
{
  int n_read;

  if (!connected) {
    ErrStr = errormsg[0];
    return (error = -1);
  }
next:
  if ((n_read = recvfrom (getdesc(), (char *) buf, len,
			  MSG_PEEK, NULL, NULL)) < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  }
  
  return n_read;              
}

int c_socket_str::poll (void *buf, const int len, const int wait_secs)
{
  int dscr = getdesc();
  int n_read;
  fd_set fdvar;
  struct timeval tiva;
  
  if (!connected) {
    ErrStr = errormsg[0];
    return (error = -1);
  }

  // call select
  FD_ZERO (&fdvar);
  FD_SET  (dscr, &fdvar);
  tiva.tv_sec  = wait_secs;
  tiva.tv_usec = 0;
  
  n_read = select (dscr+1, FT_SETTYPE(&fdvar), NULL, NULL, &tiva);
  if (n_read >= 1 && FD_ISSET (dscr, &fdvar)) {   
    n_read = ::read (dscr, (char *) buf, len);
    if (n_read < 0) {
      ErrStr = strerror (error = errno);
      return -1;
    } 
    else if (n_read == 0) {
      ErrStr = "no data read; possibly lost connection";
      error = 1;
    }
    return n_read;
  }
  
  ErrStr = "no data ready for reading";
  return -2;
}


int c_socket_str::write (const void *buf, const int len)
{
  int n_write;
  
  if (!connected) {
    ErrStr = errormsg[0];
    return (error = -1);
  }
next:
  if ((n_write = ::write (getdesc(), (const char *) buf, len)) < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  }
  
  return n_write;
}

/* -- METHODS: c_socket_dgr ----------------------------------------------- */

c_socket_dgr::c_socket_dgr() 
	: c_Socket (SOCK_DGRAM), peer()
{
  c_address addr;
  r_flag = w_flag = 0;
  
  if (bind (getdesc(), 
	    (struct sockaddr *) &addr.s_address, 
	    addr.s_addrlen) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }
  r_flag = 1;
}

c_socket_dgr::c_socket_dgr (const int desc)
	: c_Socket (desc, 0), peer()
{
  w_flag  = 1;
  r_flag  = 1;
}

c_socket_dgr::c_socket_dgr (c_address &addr) 
	: c_Socket (SOCK_DGRAM), peer()
{
  r_flag = w_flag = 0;
  
  if (bind (getdesc(), 
	    (struct sockaddr *) &addr.s_address,
	    addr.s_addrlen) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }
  
  if (getladdr (addr) < 0) {
    ErrStr = strerror (error = errno);
    return;
  }

  r_flag = 1;
}


int c_socket_dgr::read (void *buf, const int len)
{
  int n_read;
  
  if (!r_flag) {
    error  = 8989;
    ErrStr = errormsg[2];
    return -1;
  }

next:
  if ((n_read = recvfrom (getdesc(), (char *) buf, len, 0,
			  (struct sockaddr *) &peer.s_address,
			  &peer.s_addrlen)) < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  }
  
  w_flag = 1;
  return n_read;              
}

int c_socket_dgr::peek (void *buf, const int len, c_address &addr)
{
  int n_read;

  if (!r_flag) {
    error  = 8989;
    ErrStr = errormsg[2];
    return -1;
  }

next:
  if ((n_read = recvfrom (getdesc(), (char *) buf, len, MSG_PEEK,
			  (struct sockaddr *) &addr.s_address,
			  &addr.s_addrlen)) < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  }
  return n_read;
}

int c_socket_dgr::poll (void *buf, const int len, const int wait_secs)
{
  struct timeval tiva;
  int dscr = getdesc();
  int n_read;
  fd_set fdvar;
  
  if (!r_flag) {
    error  = 8989;
    ErrStr = errormsg[2];
    return -1;
  }

  // call select
  FD_ZERO (&fdvar);
  FD_SET (dscr, &fdvar);
  tiva.tv_sec = wait_secs;
  tiva.tv_usec = 0;
  
  n_read = select (dscr+1, FT_SETTYPE(&fdvar), NULL, NULL, &tiva);
  if (n_read >= 1 && FD_ISSET (dscr, &fdvar)) {   
    n_read = recvfrom (dscr, (char *) buf, len, 0,
		       (struct sockaddr *) &peer.s_address,
		       &peer.s_addrlen);
    if (n_read < 0) {
      ErrStr = strerror (error = errno);
      return -1;
    }
    w_flag = 1;
    return n_read;
  }
  
  ErrStr = "no data ready for reading";
  return -2;
}


int c_socket_dgr::write (const void *buf, const int len)
{
  int n_write;
  
  if (!w_flag) {
    error  = 2787;
    ErrStr = errormsg[3];
    return -1;
  }
  
next:
  if ((n_write = sendto (getdesc(), (char *) buf, len, 0, 
			 (struct sockaddr *) &peer.s_address,
			 peer.s_addrlen)) < 0) {
    if (errno == EINTR && getintrstatus())
      goto next;
    ErrStr = strerror (error = errno);
    return -1;
  }
  
  r_flag = 1;
  return n_write;
}


int c_socket_dgr::read (void *buf, const int len, c_address &addr)
{
  int n_read = read (buf, len);
  addr = peer;
  return n_read;
}


int c_socket_dgr::poll (void *buf, const int len, c_address &addr, 
			const int wait_secs)
{
  int n_read = poll (buf, len, wait_secs);
  addr = peer;
  return n_read;
}


int c_socket_dgr::write (const void *buf, const int len, c_address &addr)
{
  peer = addr;
  w_flag = 1;
  int n_write = write (buf, len);
  addr = peer;
  return n_write;
}
