/* ------------------------------------------------------------------------ */
/*
 *
 *  socket_class.h
 *  
 *  modified:	changed wait time for poll to 1 sec
 * 		changed set_peer (Rolf; 11.1.93)
 *
 * $Id: socket_class.h,v 1.4 1997/12/16 16:44:00 frank Exp $
 */

#ifndef SOCKET_CLASS_H
#define SOCKET_CLASS_H

/*
 * needed to understand the declarations
 */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "address_class.h"

/* -- DEFINE -------------------------------------------------------------- */

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN    31
#endif


/* -- CLASSES ------------------------------------------------------------- */
/*
 *  Base class for sockets
 *
 */
enum t_intr { norepeat, repeat };

class c_Socket {
  public:
    int			operator!(void)			const;
    int			fail(void)			const;
    const char *	whatswrong(void)		const;
    t_intr		getintrstatus(void)		const;
    t_intr		setintrstatus(const t_intr);
    int			getdesc(void)			const;

    virtual int 	read  (void *, const int) 	     = 0;
    virtual int		poll  (void *, const int, const int) = 0;
    virtual int		write (const void *, const int)      = 0;

    void 		close ();
    virtual 		~c_Socket();

    static const char *gethostname();
    int getladdr (c_address &);

  private:  
    int desc;
    t_intr intrstatus;  
    friend class c_SocketSet;
    
  protected:   
    c_Socket    (const int, const int);
    c_Socket    (const int);
    int setdesc (const int);

    int error;
    const char *ErrStr;
};

inline int
c_Socket::operator!(void) const
{
  return (error ? 1 : 0);
}

inline int
c_Socket::fail (void) const
{
  return error;
}

inline const char *
c_Socket::whatswrong (void) const
{
  return ErrStr;
}

inline t_intr
c_Socket::getintrstatus (void) const
{
  return intrstatus;
}

inline t_intr
c_Socket::setintrstatus (const t_intr newstatus)
{
  return intrstatus = newstatus;
}

inline int
c_Socket::getdesc (void) const
{
  return desc;
}

inline int
c_Socket::setdesc (const int newdesc)
{
  return desc = newdesc;
}

inline void
c_Socket::close (void)
{
  if (desc != -1)
    ::close (desc);
  desc = -1;
}

inline
c_Socket::~c_Socket ()
{
  close ();
}

/* ------------------------------------------------------------------------ */
/*
 *  Class for stream sockets
 *
 */
class c_socket_str : public c_Socket {
  public:
    c_socket_str ();					// client only
    c_socket_str (c_address &);				// client, server
    c_socket_str (const c_socket_str &);
    ~c_socket_str();

    int 		connect (const c_address &); 	// client
    c_socket_str * 	accept (void);			// server

    virtual int 	read  (void *, const int);
    virtual int 	peek  (void *, const int);
    virtual int 	poll  (void *, const int, const int=1);
    virtual int 	write (const void *, const int);

    int			isconnected(void)		const;
    const c_address *	getpeeraddr(void)		const;

  private:
    int connected;
    c_address *p_peer;  
    c_socket_str (const c_address &, const int, const int);  
};

inline int
c_socket_str::isconnected (void) const
{
  return connected;
}

inline const c_address *
c_socket_str::getpeeraddr (void) const
{
  return p_peer;
}

/* ------------------------------------------------------------------------ */
/*
 *  Class for datagram sockets
 *
 */
class c_socket_dgr : public c_Socket {
  public:
    c_socket_dgr  ();
    c_socket_dgr  (const int);
    c_socket_dgr  (c_address &);
    ~c_socket_dgr ();

    virtual int 	read  (void *, const int);
    virtual int 	poll  (void *, const int, const int=1);
    virtual int 	write (const void *, const int);

    int 		read  (void *, const int, c_address &);
    int 		peek  (void *, const int, c_address &);
    int			poll  (void *, const int, c_address &, const int=1);
    int 		write (const void *, const int, c_address &);

    const c_address *	getpeeraddr (void) 		const;
    void 		getpeer (c_address &)		const;
    void 		setpeer (const c_address &);

  private:
    c_address peer;
    int r_flag, w_flag;
};

inline const c_address *
c_socket_dgr::getpeeraddr (void) const
{
  return &peer;
}

inline void
c_socket_dgr::getpeer (c_address &addr) const
{
  addr = peer;
}

inline void
c_socket_dgr::setpeer (const c_address &addr)
{
  r_flag = 1;
  w_flag = 1;
  peer   = addr;
}

inline
c_socket_dgr::~c_socket_dgr ()
{
  close();
}

#endif // SOCKET_CLASS_H
