/* ------------------------------------------------------------------------ */
/*
 *
 *  socket_mc_class.C
 *
 */
 

/* -- INCLUDE ------------------------------------------------------------- */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#ifdef TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_SYS_SELECT_H
#include <sys/select.h>
#endif
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "address_class.h"
#include "socket_class.h"
#include "socket_mc_class.h"

/*
 * HP/UX braucht mal wieder eine Extrawurst
 */

#ifdef __hpux__
#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#define FT_SETTYPE(s)	((int *) s)
#endif
#endif
#endif
#ifndef FT_SETTYPE
#define FT_SETTYPE(s)	(s)
#endif

char * socket_mc_id = "$Id: socket_mc_class.C,v 1.3 1998/01/27 15:17:57 frank Exp $";

/* -- METHODS: c_socket_mc ----------------------------------------------- */


// Konstruktor ohne Adressuebergabe. Multicastadresse wird von Funktion
// generateaddress erzeugt.
c_socket_mc::c_socket_mc(int local)
    : c_Socket (SOCK_DGRAM), group_addr()
{ 
  sockaddr_in waddr;
    c_address addr;
    int one = 1;
    r_flag = w_flag = 0;


    // Generieren einer Internet-Multicast-Adresse
    generateaddress(addr,local);
  
    // Erzeugen einer eigenen Schreibsocket, initiale TTL auf 2

    if ((write_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      error  = errno;
      ErrStr = strerror (errno);
      return;
    }

    /*
     * Binde den Write-Socket
     */

    memset (&waddr, 0, sizeof (sockaddr_in));
    waddr.sin_family      = AF_INET;
    waddr.sin_port        = htons (0);
    waddr.sin_addr.s_addr = htonl (INADDR_ANY);
    
    if (bind (write_fd, (struct sockaddr *) &waddr, sizeof (sockaddr_in)) != 0) {
      error  = errno;
      ErrStr = strerror (error = errno);
      return;
    }

    u_char two = 2;
    setsockopt (write_fd, IPPROTO_IP, IP_MULTICAST_TTL,
		(char *)&two, sizeof(two) );

    // Aufbereiten der Addressklasse zum Gebrauch mit bind und sendto
    group_addr = addr;

    // Bem.: Beim Binden an eine Gruppenadresse kann die urspruengliche Adresse
    //       ueberschrieben werden. Zum Gebrauch der setsockopt-Aufrufe und
    //       get_group() wird aber noch die urspruengliche Adresse benoetigt.

    // Binden der Lesesocket, es findet ein zweifacher Versuch statt:
    // Bei ersten Versuch wird versucht, mit der konkreten Gruppenadresse
    // zu binden. Dies funktioniert z.B. auf Alphas, die Mips konnen jedoch
    // keine MCAST-Addressen binden. Fuer solchen Architekturen findet
    // ein zweiter Bind-Versuch mit INADR_ANY statt
    if (bind (getdesc(),(struct sockaddr *) &addr.s_address, addr.s_addrlen) == -1 )
    {
        addr.s_address.sin_addr.s_addr = INADDR_ANY;
        if (bind (getdesc(),(struct sockaddr *) &addr.s_address, addr.s_addrlen) == -1 )
        {
	  error  = errno;
	  ErrStr = strerror (errno);
	  return;
        }
    }

    setsockopt(getdesc(), SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one) );

#ifdef SO_REUSEPORT
    setsockopt(getdesc(), SOL_SOCKET, SO_REUSEPORT, (char*)&one, sizeof(one));
#endif

    // *** r_flag = 1;
    w_flag = 1;

}


c_socket_mc::c_socket_mc (c_address &addr)
    : c_Socket (SOCK_DGRAM), group_addr(addr.getinetaddr(), addr.getport())
{
  sockaddr_in waddr;

    r_flag = w_flag = 0;
    c_address tmp_addr = addr;
  
    // Checken, ob gueltige Multicast-Addresse
    if (IN_MULTICAST(ntohl(addr.s_address.sin_addr.s_addr)) != 1)
    {
      // keine Multicast-Addresse
      error  = EADDRNOTAVAIL;
      ErrStr = strerror (error);
      return;
    }

    group_addr = tmp_addr;
   
    // Erzeugen einer eigenen Schreibsocket, initiale TTL auf 2
    if ((write_fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
      error  = errno;
      ErrStr = strerror (error);
      return;
    }

    /*
     * Binde den Write-Socket
     */

    memset (&waddr, 0, sizeof (sockaddr_in));
    waddr.sin_family      = AF_INET;
    waddr.sin_port        = htons (0);
    waddr.sin_addr.s_addr = htonl (INADDR_ANY);
    
    if (bind (write_fd, (struct sockaddr *) &waddr, sizeof (sockaddr_in)) != 0) {
      error  = errno;
      ErrStr = strerror (error = errno);
      return;
    }

    u_char two = 2;
    setsockopt (write_fd, IPPROTO_IP, IP_MULTICAST_TTL,
		(char *)&two, sizeof(two) );

    // Bem.: Beim Binden an eine Gruppenadresse kann die urspruengliche Adresse
    //       ueberschrieben werden. Zum Gebrauch der setsockopt-Aufrufe und
    //       getgroup() wird aber noch die urspruengliche Adresse benoetigt.

    int one = 1;
    setsockopt (getdesc(), SOL_SOCKET, SO_REUSEADDR,
		(char *) &one, sizeof(one));
#ifdef SO_REUSEPORT
    setsockopt(getdesc(), SOL_SOCKET, SO_REUSEPORT,
	       (char *) &one, sizeof(one));
#endif
  
    // Binden der Lesesocket, es findet ein zweifacher Versuch statt:
    // Bei ersten Versuch wird versucht, mit der konkreten Gruppenadresse
    // zu binden. Dies funktioniert z.B. auf Alphas, die Mips konnen
    // jedoch keine MCAST-Addressen binden. Fuer solchen Architekturen
    // findet ein zweiter Bind-Versuch mit INADR_ANY statt

    if (bind (getdesc(),(struct sockaddr *) &tmp_addr.s_address, tmp_addr.s_addrlen) == -1 )
    {
        tmp_addr.s_address.sin_addr.s_addr = INADDR_ANY;
        if (bind (getdesc(),(struct sockaddr *) &tmp_addr.s_address, tmp_addr.s_addrlen) == -1 )
        {
	  error  = errno;
	  ErrStr = strerror (errno);
	  return;
        }
    }
  
    r_flag = 1;
    w_flag = 1; // MODIFIKATION KLAUS MICHEL
}

// Methoden zum Lesen und Schreiben 
int c_socket_mc::read (void *buf, int len)
{
    int n_read;

    if (!r_flag) {
      error  = EBADF;
      ErrStr = strerror (error);
      return -1;
    }
   
  next:
    if ((n_read = recvfrom(getdesc(), (char *) buf, len, 0,
                           (struct sockaddr *) &peer.s_address,
			   &peer.s_addrlen )) < 0)
    {
        if (errno == EINTR && getintrstatus())
            goto next;
        ErrStr = strerror (error = errno);
        return -1;
    }
  
    w_flag = 1;
    
    return n_read;
}

int c_socket_mc::read (void *buf, const int len, c_address &addr)
{
  int n_read = read (buf, len);
  addr = peer;
  return n_read;
}

int c_socket_mc::poll (void *buf, int len, int wait_secs)
{
    int n_read;
  
    if (!r_flag)
    {
      error  = EBADF;
      ErrStr = strerror (error);
      return -1;
    }

    int dscr = getdesc();
    
    // call select
    fd_set fdvar;
    timeval tiva;

    FD_ZERO (&fdvar);
    FD_SET (dscr, &fdvar);
    tiva.tv_sec = wait_secs;
    tiva.tv_usec = 0;

    n_read = select (dscr+1, FT_SETTYPE(&fdvar), NULL, NULL, &tiva);

    if (n_read >= 1 && FD_ISSET (dscr, &fdvar))
    {
        n_read = recvfrom (dscr, (char *) buf, len, 0,
                           ( struct sockaddr*)&peer.s_address, &peer.s_addrlen);
        if (n_read < 0)
        {
            ErrStr = strerror (error = errno);
            return -1;
        }
        w_flag = 1;
        return n_read;
    }
  
    ErrStr = "no data ready for reading";

    return -2;
}

int c_socket_mc::write (const void *buf, int len)
{
    int n_write;
  
    if (!w_flag)
    {
      error  = EBADF;
      ErrStr = strerror (error);
      return -1;
    }
      
  next:
    if ((n_write = sendto (write_fd, (char *) buf, len, 0,
                           (struct sockaddr *) &group_addr.s_address, 
			   group_addr.s_addrlen)) < 0)
    {
        if (errno == EINTR && getintrstatus())
            goto next;
        ErrStr = strerror (error = errno); 
        return -1;
    }

    r_flag = 1;

    return n_write;
    
}

// Diese Methode generiert eine Multicast-Adresse sowie eine Portnummer
// und gibt diese in addr zurueck. Wenn local gleich 1 ist, wird eine
// lokale Addresse zurueckgeliefert, bei 0 eine nichtlokale
// Multicastadresse

int c_socket_mc::generateaddress(c_address &addr, int local)
{
    //struct tms buffer;
    time_t *buffer = NULL;
    struct in_addr adresse;
    u_long range, int_address;
    int port;

	// Berechne den Bereich, aus dem Integerzahl auswaehlen
    if (local == MLOCAL)
    {
        // lokale Addresse, aus Bereich 224.0.0.0 bis  224.0.0.255
        range = 255;
    }
    else {
        // globale Addresse, aus Bereich 224.0.1.0 bis 239.255.255.255
        // in integer umgerechnet ergibt die Differenz 268435455
        range = 268435455;
    }

    // Initialisiere Generator mit Systemzeit, generiere Zufallszahl
    srand((int)time(buffer));
    u_long wert = rand();
    wert = wert % range;                // Modulo mit Wertbereich
        
    // Aufaddieren des Wertes auf 1. Multicastadresse...:
    // ( 224*256*256*256 = 3758096384 )
    int_address = 3758096384 + ((1-((local==MGLOBAL)?0:1)) * 255) + wert;

    wert = rand();
    wert = wert % 10000;	// Portnummmer aus Bereich 1 - 10000 + 1024
    port = wert + 1024;		// (+1024 damit keine Kollision mit
				// reservierten Portnummern)

    adresse.s_addr = int_address;
    adresse.s_addr = htonl(adresse.s_addr);

    // Eintrag der Werte in die uebergebene Adressenklasse
    addr.s_address.sin_addr.s_addr = htonl(int_address);
    addr.s_address.sin_port = htons(port);
    
    return(0);
}


int c_socket_mc::setgroup(const c_address &addr)
{     
    struct ip_mreq imr;
    c_address tmp_addr = addr;

    // Abmelden aus alter Multicast-Gruppe
    imr.imr_multiaddr.s_addr = inet_addr(group_addr.getinetaddr() );
    imr.imr_interface.s_addr = htonl(INADDR_ANY);
    setsockopt(getdesc(), IPPROTO_IP, IP_DROP_MEMBERSHIP, 
	       (char *)&imr, sizeof(struct ip_mreq) );
    
    // Checken, ob gueltige Multicast-Addresse
    if (IN_MULTICAST(ntohl(addr.s_address.sin_addr.s_addr)) != 1)
    {
      // keine Multicast-Addresse
      error  = EADDRNOTAVAIL;
      ErrStr = strerror (error);
      return -1;
    }

    group_addr = addr;
	// Schliessen der alten read-Socket, instantiieren einer neuen
	::close(getdesc());
	int newfd;

	if ((newfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	  error  = errno;
	  ErrStr = strerror (error);
	  return -1;
	}
	setdesc(newfd);

	int one = 1;
 	setsockopt (getdesc(), SOL_SOCKET, SO_REUSEADDR,
		    (char *)&one, sizeof(one) );

    // Zweifacher Bindeversuch, siehe Konstruktor
    if (bind(getdesc(), (struct sockaddr *) &tmp_addr.s_address, tmp_addr.s_addrlen ) == -1)
    {
        tmp_addr.s_address.sin_addr.s_addr = INADDR_ANY;
        if (bind(getdesc(), (struct sockaddr *) &tmp_addr.s_address, tmp_addr.s_addrlen ) == -1)
        {
            ErrStr = strerror (error = errno);
            return(-1);
        }
    }

    // lesen und schreiben erlauben
    r_flag = w_flag = 1;

    return 0;
    
}       


int c_socket_mc::joingroup()
{
    struct ip_mreq imr;

    // Anmelden in Multicast-Gruppe, damit wird Lesen dieser Gruppe ermoeglicht
    // Zum reinen Senden in die Gruppe nicht noetig
    imr.imr_multiaddr.s_addr = inet_addr(group_addr.getinetaddr() );
    imr.imr_interface.s_addr = htonl(INADDR_ANY);
    if ((setsockopt (getdesc(), IPPROTO_IP, IP_ADD_MEMBERSHIP, 
		     (char*) &imr,sizeof(struct ip_mreq))) == -1) {
      error  = errno;
      ErrStr = strerror (error);
      return -1;
    }

    return 0;
    
}

int c_socket_mc::joingroup(const char *interface)
{
  struct ip_mreq imr;

  // Anmelden in Multicast-Gruppe, damit wird Lesen dieser Gruppe ermoeglicht
  // Zum reinen Senden in die Gruppe nicht noetig
  imr.imr_multiaddr.s_addr = inet_addr(group_addr.getinetaddr() );
  if ((imr.imr_interface.s_addr = inet_addr (interface)) == INADDR_NONE) {
    hostent *nodep;
    if ((nodep = gethostbyname (interface)) == NULL) {
      ErrStr = "hostname unknown";
      error  = 1;
      return -1;
    }
    memcpy (&imr.imr_interface.s_addr, nodep->h_addr, nodep->h_length);
  }




  if ((setsockopt (getdesc(), IPPROTO_IP, IP_ADD_MEMBERSHIP, 
		     (char*) &imr,sizeof(struct ip_mreq))) == -1) {
    error  = errno;
    ErrStr = strerror (error);
    return -1;
  }

  return 0;
    
}

int c_socket_mc::leavegroup()
{
    struct ip_mreq imr;

    // Abmelden aus alter Multicast-Gruppe
    imr.imr_multiaddr.s_addr = inet_addr(group_addr.getinetaddr() );
    imr.imr_interface.s_addr = htonl(INADDR_ANY);
    if ((setsockopt (getdesc(), IPPROTO_IP, IP_DROP_MEMBERSHIP,
		     (char *) &imr,sizeof(struct ip_mreq) )) == -1) {
      error  = errno;
      ErrStr = strerror (error);
      return -1;
    }

    return 0;
}       
