/* ------------------------------------------------------------------------ */
/*
 *
 *  socket_mc_class.h
 *  
 * $Id: socket_mc_class.h,v 1.2 1998/01/20 09:07:38 juergen Exp $
 */

#ifndef SOCKET_MC_CLASS_H
#define SOCKET_MC_CLASS_H

/*
 * needed to understand the declarations
 */

#include "fpconfig.h"

#ifdef _REENTRANT
#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/socket.h>

#include "address_class.h"
#include "socket_class.h"

/* ------------------------------------------------------------------------ */
/*
 *  Class for multicast sockets
 *
 */

class c_socket_mc : public c_Socket {
private:
  c_address group_addr, peer;
  int r_flag, w_flag;
  int write_fd;               // Filedeskriptor fuer die Write-Socket
  
public:
  enum {
    MGLOBAL,
    MLOCAL
  };

  c_socket_mc (int = MLOCAL);
  c_socket_mc (c_address &);
  ~c_socket_mc();
  
  virtual int read  (void *, int);
  virtual int read  (void *, int, c_address &);
  virtual int poll  (void *, int, int wait_secs = 1);
  virtual int write (const void *, int);
  
  static int generateaddress(c_address &addr, int local);
  int joingroup (void);
  int joingroup (const c_address &addr);
  int joingroup (const char* interface);
  int joingroup (const c_address &addr, const char* interface);
  int leavegroup(void);
  
  int set_ttl  (u_char);
  int loopback (u_char = 1);
  
  void getgroupaddr (c_address &addr);
  int setgroup(const c_address &addr); 
  
  const c_address* getpeeraddr(void);
  void getpeer(c_address &addr);
  
  int getwritedesc(void);
};

inline
c_socket_mc::~c_socket_mc (void)
{
  ::close (write_fd);
}

inline int
c_socket_mc::set_ttl(u_char ttl)
{
  return setsockopt(write_fd, IPPROTO_IP, IP_MULTICAST_TTL,
		    (char *) &ttl, sizeof(ttl));
}

inline int
c_socket_mc::loopback (u_char flag) 
{
  return setsockopt(write_fd, IPPROTO_IP, IP_MULTICAST_LOOP,
                    (char *) &flag, sizeof(flag));
}

inline
void c_socket_mc::getgroupaddr (c_address &addr)
{
  addr = group_addr;
}

inline
int c_socket_mc::joingroup (const c_address &addr)
{
    setgroup(addr);
    return joingroup();
}

inline
int c_socket_mc::joingroup (const c_address &addr, const char* interface)
{
    setgroup(addr);
    return joingroup(interface);
}

inline
void c_socket_mc::getpeer (c_address &addr) 
{
    addr = peer;
}

inline
const c_address* c_socket_mc::getpeeraddr()
{
    return &peer;
}

inline int
c_socket_mc::getwritedesc (void)
{
  return write_fd;
}

#endif
