/*
 * Teste Message Queue
 *
 * ./msgtest key <id> <message>
 *     Verschickt eine Message an die gegebene Key/Id
 *
 * ./msgtest key <id>
 *     Empfaengt eine Message fuer die gegebene Key/Id
 *
 * ./msgtest key
 *     Zerstoert die Message Queue
 */

#include <fpmsgqueue.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

/*
 * Unser Paket; wir verpacken einen String in einer Struktur.
 */

struct mypack {
  char string[256];
};

fpPacket<mypack> pack;

int
main (int argc, char *argv[])
{
  fpMsgQueue mq;
  key_t key;
  long id;

  if (argc == 2) {
    key = atoi (argv[1]);

    if (mq.attach (key) != 0) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }
    if (mq.destroy () != 0) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }
    printf ("Message Queue geloescht.\n");
  }
  else if (argc == 3) {
    key = atoi (argv[1]);
    id  = atoi (argv[2]);

    if (mq.attach (key) != 0) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }

    mq.setId (id);

    if (mq.recv (&pack) == -1) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }

    printf ("Botschaft von %ld: \"%s\"\n",
	    pack.source(),
	    ((mypack *) pack)->string);

    mq.detach ();
  }
  else if (argc == 4) {
    key = atoi (argv[1]);
    id  = atoi (argv[2]);

    strcpy (((mypack *) pack)->string, argv[3]);

    if (mq.create (key) != 0) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }

    if (mq.send (&pack, id) == -1) {
      printf ("error: %s\n", strerror (mq.error()));
      return 1;
    }

    mq.detach ();
  }
  else {
    printf ("Falsches Kommando.\n");
  }
  return 0;
}

