
/*
 * Testprogramm fuer Shared Memory und Semaphore.
 *
 * Aufruf:
 *    ./shmtest <key> <nachricht>
 *           Schreibt eine Nachricht ins Shared Memory und wartet darauf,
 *           dass die Nachricht gelesen wird.
 *
 *    ./shmtest <key>
 *           Wartet, dass eine Nachricht ins Shared Memory geschrieben
 *           wird und liest diese.
 *
 * Man beachte, dass dieses Programm noch ein paar "Problemstellen"
 * enthaelt, die sich der Leser selber suchen darf.
 *
 * "I give the people what they want. Some of them want to find bugs,
 * so I added some." -- Larry Wall
 */

#include <fpshmem.h>
#include <fpsema.h>
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <string.h>

void Master (int argc, char *argv[])
{
  fpShmem shmem;
  fpSema sema;
  char *addr;

  if (shmem.create (atoi (argv[1]), 1024) == -1) {
    printf ("Kann shared Memory nicht erzeugen: %s\n",
	    shmem.strerror ());
    return;
  }

  if (sema.create  (atoi (argv[1]), 3) == -1) {
    printf ("Kann Semaphore nicht erzeugen: %s\n",
	    sema.strerror ());
    shmem.destroy ();
    return;
  }

  /*
   * Warte darauf, dass das shared Memory leer ist
   */

  sema[1].zero();

  addr = (char *) shmem;

  strcpy (addr, argv[2]);

  /*
   * Signalisiere, dass eine Nachricht im shared memory ist
   */

  sema[1]++;

  /*
   * Warte darauf, dass die Nachricht gelesen wird
   */

  sema[2]--;

  /*
   * Und fertig.
   */

  shmem.destroy ();
  sema.destroy ();
}

/*
 * Der Klient
 */

void Client (int argc, char *argv[])
{
  fpShmem shmem;
  fpSema sema;
  char *addr;

  if (shmem.attach (atoi (argv[1]), 1024) == -1) {
    printf ("Kann nicht aufs shared Memory zugreifen: %s\n",
	    shmem.strerror ());
    return;
  }

  if (sema.attach (atoi (argv[1]), 2) == -1) {
    printf ("Kann nicht auf die Semaphore zugreifen: %s\n",
	    sema.strerror ());
    shmem.detach ();
    return;
  }

  /*
   * Warte darauf, dass eine Nachricht im shared Memory abgelegt wird.
   */

  sema[1]--;

  /*
   * Lese Nachricht
   */

  addr = (char *) shmem;

  printf ("%s\n", addr);

  sema[2]++;

  /*
   * Und tschuess
   */

  shmem.detach ();
  sema.detach ();
}

/*
 * Hauptprogramm
 */

int
main (int argc, char *argv[])
{
  if (argc == 3) {
    Master (argc, argv);
  } else if (argc == 2) {
    Client (argc, argv);
  } else {
    printf ("Aufruf:\n");
    printf ("  %s <key> [<Nachricht>]\n");
  }
  return 0;
}

