/********************************************************************/
/******  thread_class.C *********************************************/
/********************************************************************/

#include "fpconfig.h"

#include <iostream.h>
#include "thread_class.h"

char * thread_id = "$Id: thread_class.C,v 1.1 1997/12/16 12:38:25 frank Exp $";

void t_cleanup(void* arg1) {
	
    c_thread* my_obj = (c_thread*)arg1; 
    pthread_mutex_lock(&(my_obj->threadmutex));
    my_obj->status = c_thread::EXITING;
    if(my_obj->detach_flag == 0){
    	pthread_mutex_unlock(&(my_obj->threadmutex));
    } else {
    	pthread_mutex_unlock(&(my_obj->threadmutex));
	my_obj->local = 1;
	delete my_obj;
    }	
    return;

};


	
// Definition der Threadfunktion:

void* threadstart(void* argument){


    thread_info* myinfo = (thread_info*)argument;
    c_thread* my_obj;
    void* myarg;
    void* result;

    // Auslesen der Parameter:
    my_obj = myinfo->obj_pointer;
    myarg  = myinfo->parameter;
    pthread_cleanup_push(t_cleanup, (void*)my_obj);

    // Aufruf der Methode 'thread_main':
    result = my_obj->thread_main(myarg);
    pthread_cleanup_pop(1);
    return result;
};


	
c_thread::c_thread(){

	exit_value = 0;
	local = 0;
	detach_flag = 0;
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_attr_init(&my_attr);	// Erzeugung des Attributes
        pthread_mutexattr_init(&mutex_attr);
        pthread_mutex_init(&threadmutex, &mutex_attr);
#else
	pthread_attr_create(&my_attr);	// Erzeugung des Attributes	
        pthread_mutexattr_create(&mutex_attr);
	pthread_mutex_init(&threadmutex, mutex_attr); // Erzeugung des Mutex
#endif
	status = INITIAL;

};


 
c_thread::c_thread(dflag tf1){

	exit_value = 0;
	local = 0;
	detach_flag = 1;
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_attr_init(&my_attr);	// Erzeugung des Attributes
        pthread_mutexattr_init(&mutex_attr);
        pthread_attr_setdetachstate(&my_attr, PTHREAD_CREATE_DETACHED);
        pthread_mutex_init(&threadmutex,&mutex_attr);
#else
	pthread_attr_create(&my_attr);	// Erzeugung des Attributes	
        pthread_mutexattr_create(&mutex_attr);
#ifdef HAVE_PTHREAD_CREATE_DETACHED
        pthread_attr_setdetach_np(&my_attr, PTHREAD_CREATE_DETACHED);
#endif
	pthread_mutex_init(&threadmutex, mutex_attr); // Erzeugung des Mutex
#endif
	status = INITIAL;
	
};


c_thread::~c_thread(){
	if(!local){
		pthread_cancel(thread);	// Terminierung der Thread
#ifdef HAVE_POSIX_THREADS_1CD10
		pthread_detach(thread);
#else
		pthread_detach(&thread);
#endif
	}
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_attr_destroy(&my_attr);
	pthread_mutexattr_destroy(&mutex_attr);
#else
	pthread_attr_delete(&my_attr);
	pthread_mutexattr_delete(&mutex_attr);
#endif

	pthread_mutex_destroy(&threadmutex);
};



int c_thread::start(void *par){
	int retval;
	// Falls Thread abgelaufen -> Loeschen der Thread, um Neustart
	// zu ermoeglichen:
	pthread_mutex_lock(&threadmutex);
#ifdef HAVE_POSIX_THREADS_1CD10
	if (status == STOPPED) pthread_detach(thread);
#else
        if (status == STOPPED) pthread_detach(&thread);
#endif
	// Falls Thread nicht bereits laeuft -> Starten der Thread:
	if ((status == INITIAL)||(status == STOPPED)){
	   thread_data.obj_pointer = this;
	   thread_data.parameter = par;
	   pthread_create(&thread,
#ifdef HAVE_POSIX_THREADS_1CD10
				&my_attr,
#else
				my_attr,
#endif
				threadstart,
				(void*)&thread_data);
	   status = RUNNING;
#ifndef HAVE_PTHREAD_CREATE_DETACHED
	if (detach_flag == 1){
		cerr << "detached thread" << endl;
#ifdef HAVE_POSIX_THREADS_1CD10
		pthread_detach(thread);
#else
		pthread_detach(&thread);
#endif
	}
#endif
	   retval = 1;
	} else {
	   cerr << "wrong thread-state: "<<status<<endl;
	   cerr << "if state == 1 : thread is still running"<<endl;
	   retval = -1;
	};
	pthread_mutex_unlock(&threadmutex);

	return retval;
};


void* c_thread::stop(){
	pthread_cancel(thread);
	pthread_join(thread,&exit_value);
	pthread_mutex_lock(&threadmutex);
	status = STOPPED;
	pthread_mutex_unlock(&threadmutex);
	return exit_value;
};


void* c_thread::join(){
	pthread_join(thread,&exit_value);
	pthread_mutex_lock(&threadmutex);
	status = STOPPED;
	pthread_mutex_unlock(&threadmutex);
	return exit_value;
};

int c_thread::getstate(){
	int retval;
	pthread_mutex_lock(&threadmutex);
	retval = status;
	pthread_mutex_unlock(&threadmutex);
	return retval;
};

int detach_object(c_thread* tp){
	pthread_mutex_lock(&(tp->threadmutex));
	if(tp->detach_flag == 1){
		pthread_mutex_unlock(&(tp->threadmutex));
		return 1;
	}
	if(tp->status == c_thread::EXITING){
		pthread_mutex_unlock(&(tp->threadmutex));
		delete tp;
		return 1;
	}
#ifdef HAVE_POSIX_THREADS_1CD10
	pthread_detach(tp->thread);
#else
	pthread_detach(&(tp->thread));
#endif
	tp->detach_flag = 1;
	pthread_mutex_unlock(&(tp->threadmutex));
	return 1;
};

