/************************************************************************/
/******  thread_class.h  ************************************************/
/************************************************************************/

#ifndef THREAD_CLASS
#define THREAD_CLASS

#include "fpconfig.h"

#ifndef _REENTRANT
#define _REENTRANT 1
#endif

#ifdef HAVE_DCE_CMA_H
#include <dce/cma.h>
#endif

#include <pthread.h>

//forward declaration

enum dflag { PREDETACHED };

class c_thread;

// An die Threadfunktion zu uebergebende Datenstruktur:
typedef struct {
	c_thread* 	obj_pointer;	// Zeiger auf das Threadobject
	void*		parameter;	// Argument fuer die virtuelle Methode
} thread_info;

class c_thread {
	friend void* threadstart(void*); // Funktion 'threadstart' darf
					// auf privaten Bereich zugreifen
	friend int detach_object(c_thread*);
	friend void t_cleanup(void*);
	
private:
	void* exit_value;
	int status;		// Ausfuehrungszustand der Thread
	// Der Mutex sichert den exklusiven Zugriff auf Daten der Thread,
	// auf die auch andere Threads zugreifen (z.B. Statusabfrage).
	thread_info thread_data;
	virtual void* thread_main(void*) = 0; // In der Thread auszufuehrende 
					    // Methode
	int local;
	int detach_flag;

protected:

	pthread_t thread;		// "thread-handle"
	pthread_attr_t my_attr;		// "attribute-handle"
	pthread_mutex_t threadmutex;	// "mutex-handle"
	pthread_mutexattr_t mutex_attr;
public:
        // Definition der Ausfuehrungszustaende:
        enum {
	  INITIAL,	// Thread wurde noch nicht gestartet
	  RUNNING,	// Thread laeuft
	  EXITING,	// Thread terminiert
	  STOPPED	// Thread ist abgelaufen oder wurde angehalten
	};

	c_thread();
	c_thread(dflag);

	virtual ~c_thread();
	int start(void*);	// startet die Thread ( void* wird an die
				// Methode 'thread_main' uebergeben)

	void* stop();		// explizite Terminierung der Thread
	pthread_t* gethandle(){ return &thread; };
	void* join();		// Warten dass die Thread terminiert
	int getstate();

};

#endif /* THREAD_CLASS */

